@@ -11,8 +11,12 @@
 #include <QAbstractEventDispatcher>
 #include <QDBusConnection>
 #include <QDBusServiceWatcher>
+#include <QGuiApplication>
+#include <QIcon>
 #include <QTimer>
 
+#include <memory>
+
 #include "card.h"
 #include "client.h"
 #include "module.h"
@@ -48,6 +52,8 @@ qint64 maximumUIVolume()
     return PA_VOLUME_UI_MAX;
 }
 
+QString ContextPrivate::s_applicationId;
+
 #ifndef K_DOXYGEN
 
 static bool isGoodState(int eol)
@@ -113,7 +119,10 @@ static void source_output_cb(pa_context *context, const pa_source_output_info *i
         return;
     // FIXME: This forces excluding these apps
     if (const char *app = pa_proplist_gets(info->proplist, PA_PROP_APPLICATION_ID)) {
-        if (strcmp(app, "org.PulseAudio.pavucontrol") == 0 || strcmp(app, "org.gnome.VolumeControl") == 0 || strcmp(app, "org.kde.kmixd") == 0)
+        if (strcmp(app, "org.PulseAudio.pavucontrol") == 0 //
+            || strcmp(app, "org.gnome.VolumeControl") == 0 //
+            || strcmp(app, "org.kde.kmixd") == 0 //
+            || strcmp(app, "org.kde.plasma-pa") == 0) //
             return;
     }
     Q_ASSERT(context);
@@ -152,6 +161,12 @@ static void server_cb(pa_context *context, const pa_server_info *info, void *dat
 {
     Q_ASSERT(context);
     Q_ASSERT(data);
+    if (!info) {
+        // info may be nullptr when e.g. the server doesn't reply in time (e.g. it is stuck)
+        // https://bugs.kde.org/show_bug.cgi?id=454647
+        qCWarning(PULSEAUDIOQT) << "server_cb() called without info!";
+        return;
+    }
     static_cast<ContextPrivate *>(data)->serverCallback(info);
 }
 
@@ -247,6 +262,11 @@ Context::Context(QObject *parent)
         d->connectToDaemon();
     });
 
+    connect(&d->m_connectTimer, &QTimer::timeout, this, [this] {
+        d->connectToDaemon();
+        d->checkConnectTries();
+    });
+
     connect(&d->m_sinks, &MapBaseQObject::added, this, [this](int, QObject *object) {
         Q_EMIT sinkAdded(static_cast<Sink *>(object));
     });
@@ -431,7 +451,8 @@ void ContextPrivate::contextStateCallback(pa_context *c)
     qCDebug(PULSEAUDIOQT) << "state callback";
     pa_context_state_t state = pa_context_get_state(c);
     if (state == PA_CONTEXT_READY) {
-        qCDebug(PULSEAUDIOQT) << "ready";
+        qCDebug(PULSEAUDIOQT) << "ready, stopping connect timer";
+        m_connectTimer.stop();
 
         // 1. Register for the stream changes (except during probe)
         if (m_context == c) {
@@ -502,9 +523,8 @@ void ContextPrivate::contextStateCallback(pa_context *c)
             m_context = nullptr;
         }
         reset();
-        QTimer::singleShot(1000, q, [this] {
-            connectToDaemon();
-        });
+        qCDebug(PULSEAUDIOQT) << "Starting connect timer";
+        m_connectTimer.start(std::chrono::seconds(5));
     }
 }
 
@@ -645,7 +665,17 @@ void ContextPrivate::connectToDaemon()
 
     pa_mainloop_api *api = pa_glib_mainloop_get_api(m_mainloop);
     Q_ASSERT(api);
-    m_context = pa_context_new(api, "QPulse");
+
+    pa_proplist *proplist = pa_proplist_new();
+    pa_proplist_sets(proplist, PA_PROP_APPLICATION_NAME, QGuiApplication::applicationDisplayName().toUtf8().constData());
+    if (!s_applicationId.isEmpty()) {
+        pa_proplist_sets(proplist, PA_PROP_APPLICATION_ID, s_applicationId.toUtf8().constData());
+    } else {
+        pa_proplist_sets(proplist, PA_PROP_APPLICATION_ID, QGuiApplication::desktopFileName().toUtf8().constData());
+    }
+    pa_proplist_sets(proplist, PA_PROP_APPLICATION_ICON_NAME, QGuiApplication::windowIcon().name().toUtf8().constData());
+    m_context = pa_context_new_with_proplist(api, nullptr, proplist);
+    pa_proplist_free(proplist);
     Q_ASSERT(m_context);
 
     if (pa_context_connect(m_context, NULL, PA_CONTEXT_NOFAIL, nullptr) < 0) {
@@ -658,6 +688,14 @@ void ContextPrivate::connectToDaemon()
     pa_context_set_state_callback(m_context, &context_state_callback, this);
 }
 
+void ContextPrivate::checkConnectTries()
+{
+    if (++m_connectTries == 5) {
+        qCWarning(PULSEAUDIOQT) << "Giving up after" << m_connectTries << "tries to connect";
+        m_connectTimer.stop();
+    }
+}
+
 void ContextPrivate::reset()
 {
     m_sinks.reset();
@@ -669,6 +707,7 @@ void ContextPrivate::reset()
     m_modules.reset();
     m_streamRestores.reset();
     m_server->reset();
+    m_connectTries = 0;
 }
 
 bool Context::isValid()
@@ -788,4 +827,36 @@ void ContextPrivate::setGenericDeviceForStream(
     }
 }
 
+void ContextPrivate::setGenericVolumes(
+    quint32 index,
+    QVector<qint64> channelVolumes,
+    pa_cvolume cVolume,
+    const std::function<pa_operation *(pa_context *, uint32_t, const pa_cvolume *, pa_context_success_cb_t, void *)> &pa_set_volume)
+{
+    if (!m_context) {
+        return;
+    }
+    Q_ASSERT(channelVolumes.count() == cVolume.channels);
+
+    pa_cvolume newCVolume = cVolume;
+    for (int i = 0; i < channelVolumes.count(); ++i) {
+        newCVolume.values[i] = qBound<qint64>(0, channelVolumes.at(i), PA_VOLUME_MAX);
+    }
+
+    if (!PAOperation(pa_set_volume(m_context, index, &newCVolume, nullptr, nullptr))) {
+        qCWarning(PULSEAUDIOQT) << "pa_set_volume failed";
+        return;
+    }
+}
+
+void Context::setApplicationId(const QString &applicationId)
+{
+    ContextPrivate::s_applicationId = applicationId;
+}
+
+pa_context *Context::context() const
+{
+    return d->m_context;
+}
+
 } // PulseAudioQt
