@@ -17,6 +17,7 @@ Profile::Profile(QObject *parent)
 
 Profile::~Profile()
 {
+    delete d;
 }
 
 ProfilePrivate::ProfilePrivate(Profile *q)
@@ -43,4 +44,14 @@ Profile::Availability Profile::availability() const
     return d->m_availability;
 }
 
+quint32 Profile::sinkCount() const
+{
+    return d->m_sinkCount;
+}
+
+quint32 Profile::sourceCount() const
+{
+    return d->m_sourceCount;
+}
+
 } // PulseAudioQt
