@@ -29,6 +29,7 @@ Server::Server(Context *context)
 
 Server::~Server()
 {
+    delete d;
 }
 
 ServerPrivate::ServerPrivate(Server *q)
@@ -82,6 +83,13 @@ void ServerPrivate::update(const pa_server_info *info)
     m_defaultSinkName = QString::fromUtf8(info->default_sink_name);
     m_defaultSourceName = QString::fromUtf8(info->default_source_name);
 
+    const bool isPw = QString::fromUtf8(info->server_name).contains("PipeWire");
+
+    if (isPw != m_isPipeWire) {
+        m_isPipeWire = isPw;
+        Q_EMIT q->isPipeWireChanged();
+    }
+
     q->updateDefaultDevices();
 }
 
@@ -121,4 +129,9 @@ void Server::updateDefaultDevices()
     }
 }
 
+bool Server::isPipeWire() const
+{
+    return d->m_isPipeWire;
+}
+
 } // PulseAudioQt
