@@ -58,4 +58,9 @@ void SinkInput::setChannelVolume(int channel, qint64 volume)
     Context::instance()->d->setGenericVolume(index(), channel, volume, VolumeObject::d->cvolume(), &pa_context_set_sink_input_volume);
 }
 
+void SinkInput::setChannelVolumes(const QVector<qint64> &channelVolumes)
+{
+    Context::instance()->d->setGenericVolumes(index(), channelVolumes, VolumeObject::d->m_volume, &pa_context_set_sink_input_volume);
+}
+
 } // PulseAudioQt
