@@ -27,6 +27,7 @@ StreamRestore::StreamRestore(quint32 index, const QVariantMap &properties, QObje
 
 StreamRestore::~StreamRestore()
 {
+    delete d;
 }
 
 StreamRestorePrivate::StreamRestorePrivate(StreamRestore *q)
@@ -52,12 +53,12 @@ void StreamRestorePrivate::update(const pa_ext_stream_restore_info *info)
         m_muted = info->mute;
         Q_EMIT q->mutedChanged();
     }
-    if (memcmp(&m_volume, &info->volume, sizeof(pa_cvolume)) != 0) {
+    if (!pa_cvolume_equal(&m_volume, &info->volume)) {
         m_volume = info->volume;
         Q_EMIT q->volumeChanged();
         Q_EMIT q->channelVolumesChanged();
     }
-    if (memcmp(&m_channelMap, &info->channel_map, sizeof(pa_channel_map)) != 0) {
+    if (!pa_channel_map_equal(&m_channelMap, &info->channel_map)) {
         m_channels.clear();
         m_channels.reserve(info->channel_map.channels);
         for (int i = 0; i < info->channel_map.channels; ++i) {
@@ -94,6 +95,13 @@ qint64 StreamRestore::volume() const
 void StreamRestore::setVolume(qint64 volume)
 {
     pa_cvolume vol = d->m_cache.valid ? d->m_cache.volume : d->m_volume;
+
+    // If no channel exists force one. We need one to be able to control the volume
+    // See https://bugs.kde.org/show_bug.cgi?id=407397
+    if (vol.channels == 0) {
+        vol.channels = 1;
+    }
+
     for (int i = 0; i < vol.channels; ++i) {
         vol.values[i] = volume;
     }
@@ -184,6 +192,13 @@ void StreamRestorePrivate::writeChanges(const pa_cvolume &volume, bool muted, co
     info.device = deviceData.isEmpty() ? nullptr : deviceData.constData();
     info.mute = muted;
 
+    // If no channel exists force one. We need one to be able to control the volume
+    // See https://bugs.kde.org/show_bug.cgi?id=407397
+    if (info.channel_map.channels == 0) {
+        info.channel_map.channels = 1;
+        info.channel_map.map[0] = PA_CHANNEL_POSITION_MONO;
+    }
+
     m_cache.valid = true;
     m_cache.volume = volume;
     m_cache.muted = muted;
